"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const chalk_1 = require("chalk");
const execa = require("execa");
const fs = require("fs-extra");
const _ = require("lodash");
const path = require("path");
const line_transform_1 = require("../line_transform");
const screen = require("../screen");
const YML = require('js-yaml');
const wrapAnsi = require('wrap-ansi');
const Debug = require('debug');
const stringWidth = require('string-width');
let port = 5000;
let colorIdx = 0;
let maxHeaderLength = 0;
function getColor() {
    const color = ['blueBright', 'greenBright'][colorIdx++];
    if (!color) {
        colorIdx = 0;
        return getColor();
    }
    return chalk_1.default[color];
}
class Dev extends command_1.Command {
    async run() {
        this.parse(Dev);
        const yml = YML.safeLoad(await fs.readFile('Meepfile.yml', 'utf8'));
        this.debug('yml:');
        this.debug(yml);
        // TODO: find a better way to not require sorting
        const components = Object.entries(yml.components || {});
        maxHeaderLength = _.max(components.map(([name]) => name.length)) || 0;
        const procs = components.map(([name, c]) => this.start(name, c));
        await Promise.all(procs);
    }
    async start(name, c) {
        const env = {
            PORT: (port++).toString()
        };
        const debug = Debug(`meep:${name}`);
        const root = path.join(process.cwd(), name);
        const header = getColor()(name.padEnd(maxHeaderLength));
        let command = c.command;
        if (!command) {
            switch (c.type) {
                case 'node':
                    command = 'npm start';
                    break;
                case 'static':
                    if (c.spa)
                        command = 'meep serve --spa';
                    else
                        command = 'meep serve';
                    break;
                default:
                    throw new Error(`unexpected type: ${c.type} on ${name}`);
            }
        }
        debug(command);
        const proc = execa.shell(command, {
            cwd: root,
            encoding: 'utf8',
            env,
        });
        for (let stream of ['stdout', 'stderr']) {
            proc[stream]
                .pipe(new line_transform_1.default())
                .setEncoding('utf8')
                .on('data', output => {
                const maxWidth = screen[stream] || 120;
                let lines = wrapAnsi(output, maxWidth - stringWidth(header), {
                    hard: true
                }).split('\n').map((s) => `${header} ${s}`);
                if (!lines.length)
                    return;
                process[stream].write(lines.join('\n') + '\n');
            });
        }
        proc.on('close', code => this.log(`${header} exited with code ${code}`));
        return proc;
    }
}
Dev.flags = {
    help: command_1.flags.help({ char: 'h' }),
};
exports.default = Dev;
